@@ -1,11 +1,8 @@
-import asyncio
-import io
-import tempfile
 import typing
 from enum import Enum
-from urllib.parse import unquote
+from urllib.parse import unquote_plus
 
-from starlette.datastructures import Headers
+from starlette.datastructures import FormData, Headers, UploadFile
 
 try:
     from multipart.multipart import parse_options_header
@@ -34,31 +31,6 @@ class MultiPartMessage(Enum):
     END = 8
 
 
-class UploadFile:
-    def __init__(self, filename: str) -> None:
-        self.filename = filename
-        self._file = io.BytesIO()  # type: typing.IO[typing.Any]
-        self._loop = asyncio.get_event_loop()
-
-    def create_tempfile(self) -> None:
-        self._file = tempfile.SpooledTemporaryFile()
-
-    async def setup(self) -> None:
-        await self._loop.run_in_executor(None, self.create_tempfile)
-
-    async def write(self, data: bytes) -> None:
-        await self._loop.run_in_executor(None, self._file.write, data)
-
-    async def read(self, size: int = None) -> bytes:
-        return await self._loop.run_in_executor(None, self._file.read, size)
-
-    async def seek(self, offset: int) -> None:
-        await self._loop.run_in_executor(None, self._file.seek, offset)
-
-    async def close(self) -> None:
-        await self._loop.run_in_executor(None, self._file.close)
-
-
 class FormParser:
     def __init__(
         self, headers: Headers, stream: typing.AsyncGenerator[bytes, None]
@@ -90,7 +62,7 @@ class FormParser:
         message = (FormMessage.END, b"")
         self.messages.append(message)
 
-    async def parse(self) -> typing.Dict[str, typing.Union[str, UploadFile]]:
+    async def parse(self) -> FormData:
         # Callbacks dictionary.
         callbacks = {
             "on_field_start": self.on_field_start,
@@ -105,7 +77,9 @@ class FormParser:
         field_name = b""
         field_value = b""
 
-        result = {}  # type: typing.Dict[str, typing.Union[str, UploadFile]]
+        items = (
+            []
+        )  # type: typing.List[typing.Tuple[str, typing.Union[str, UploadFile]]]
 
         # Feed the parser with data from the request.
         async for chunk in self.stream:
@@ -124,13 +98,13 @@ class FormParser:
                 elif message_type == FormMessage.FIELD_DATA:
                     field_value += message_bytes
                 elif message_type == FormMessage.FIELD_END:
-                    result[field_name.decode("latin-1")] = unquote(
-                        field_value.decode("latin-1")
-                    )
+                    name = unquote_plus(field_name.decode("latin-1"))
+                    value = unquote_plus(field_value.decode("latin-1"))
+                    items.append((name, value))
                 elif message_type == FormMessage.END:
                     pass
 
-        return result
+        return FormData(items)
 
 
 class MultiPartParser:
@@ -176,7 +150,7 @@ class MultiPartParser:
         message = (MultiPartMessage.END, b"")
         self.messages.append(message)
 
-    async def parse(self) -> typing.Dict[str, typing.Union[str, UploadFile]]:
+    async def parse(self) -> FormData:
         # Parse the Content-Type header to get the multipart boundary.
         content_type, params = parse_options_header(self.headers["Content-Type"])
         boundary = params.get(b"boundary")
@@ -202,7 +176,9 @@ class MultiPartParser:
         data = b""
         file = None  # type: typing.Optional[UploadFile]
 
-        result = {}  # type: typing.Dict[str, typing.Union[str, UploadFile]]
+        items = (
+            []
+        )  # type: typing.List[typing.Tuple[str, typing.Union[str, UploadFile]]]
 
         # Feed the parser with data from the request.
         async for chunk in self.stream:
@@ -229,7 +205,8 @@ class MultiPartParser:
                     if b"filename" in options:
                         filename = options[b"filename"].decode("latin-1")
                         file = UploadFile(filename=filename)
-                        await file.setup()
+                    else:
+                        file = None
                 elif message_type == MultiPartMessage.PART_DATA:
                     if file is None:
                         data += message_bytes
@@ -237,12 +214,12 @@ class MultiPartParser:
                         await file.write(message_bytes)
                 elif message_type == MultiPartMessage.PART_END:
                     if file is None:
-                        result[field_name] = data.decode("latin-1")
+                        items.append((field_name, data.decode("latin-1")))
                     else:
                         await file.seek(0)
-                        result[field_name] = file
+                        items.append((field_name, file))
                 elif message_type == MultiPartMessage.END:
                     pass
 
         parser.finalize()
-        return result
+        return FormData(items)
