@@ -3,7 +3,8 @@ import io
 import sys
 import typing
 
-from starlette.types import ASGIApp, ASGIInstance, Message, Receive, Scope, Send
+from starlette.concurrency import run_in_threadpool
+from starlette.types import ASGIInstance, Message, Receive, Scope, Send
 
 
 def build_environ(scope: Scope, body: bytes) -> dict:
@@ -15,7 +16,7 @@ def build_environ(scope: Scope, body: bytes) -> dict:
         "SCRIPT_NAME": "",
         "PATH_INFO": scope["path"],
         "QUERY_STRING": scope["query_string"].decode("ascii"),
-        "SERVER_PROTOCOL": "HTTP/%s" % scope["http_version"],
+        "SERVER_PROTOCOL": f"HTTP/{scope['http_version']}",
         "wsgi.version": (1, 0),
         "wsgi.url_scheme": scope.get("scheme", "http"),
         "wsgi.input": io.BytesIO(body),
@@ -42,7 +43,7 @@ def build_environ(scope: Scope, body: bytes) -> dict:
         elif name == "content-type":
             corrected_name = "CONTENT_TYPE"
         else:
-            corrected_name = "HTTP_%s" % name.upper().replace("-", "_")
+            corrected_name = f"HTTP_{name}".upper().replace("-", "_")
         # HTTPbis say only ASCII chars are allowed in headers, but we latin1 just in case
         value = value.decode("latin1")
         if corrected_name in environ:
@@ -80,14 +81,19 @@ class WSGIResponder:
             body += message.get("body", b"")
             more_body = message.get("more_body", False)
         environ = build_environ(self.scope, body)
-        wsgi = self.loop.run_in_executor(None, self.wsgi, environ, self.start_response)
-        sender = self.loop.create_task(self.sender(send))
-        await asyncio.wait_for(wsgi, None)
-        self.send_queue.append(None)
-        self.send_event.set()
-        await asyncio.wait_for(sender, None)
-        if self.exc_info is not None:
-            raise self.exc_info[0].with_traceback(self.exc_info[1], self.exc_info[2])
+        try:
+            sender = self.loop.create_task(self.sender(send))
+            await run_in_threadpool(self.wsgi, environ, self.start_response)
+            self.send_queue.append(None)
+            self.send_event.set()
+            await asyncio.wait_for(sender, None)
+            if self.exc_info is not None:
+                raise self.exc_info[0].with_traceback(
+                    self.exc_info[1], self.exc_info[2]
+                )
+        finally:
+            if not sender.done():
+                sender.cancel()  # pragma: no cover
 
     async def sender(self, send: Send) -> None:
         while True:
