@@ -1,10 +1,10 @@
+import asyncio
 import http.cookies
 import json
 import typing
 from collections.abc import Mapping
-from urllib.parse import unquote
 
-from starlette.datastructures import URL, Headers, QueryParams
+from starlette.datastructures import URL, Address, FormData, Headers, QueryParams
 from starlette.formparsers import FormParser, MultiPartParser
 from starlette.types import Message, Receive, Scope
 
@@ -18,16 +18,15 @@ class ClientDisconnect(Exception):
     pass
 
 
-async def empty_receive() -> Message:
-    raise RuntimeError("Receive channel has not been made available")
+class HTTPConnection(Mapping):
+    """
+    A base class for incoming HTTP connections, that is used to provide
+    any functionality that is common to both `Request` and `WebSocket`.
+    """
 
-
-class Request(Mapping):
     def __init__(self, scope: Scope, receive: Receive = None) -> None:
-        assert scope["type"] == "http"
+        assert scope["type"] in ("http", "websocket")
         self._scope = scope
-        self._receive = empty_receive if receive is None else receive
-        self._stream_consumed = False
 
     def __getitem__(self, key: str) -> str:
         return self._scope[key]
@@ -38,10 +37,6 @@ class Request(Mapping):
     def __len__(self) -> int:
         return len(self._scope)
 
-    @property
-    def method(self) -> str:
-        return self._scope["method"]
-
     @property
     def url(self) -> URL:
         if not hasattr(self, "_url"):
@@ -57,7 +52,7 @@ class Request(Mapping):
     @property
     def query_params(self) -> QueryParams:
         if not hasattr(self, "_query_params"):
-            self._query_params = QueryParams(scope=self._scope)
+            self._query_params = QueryParams(self._scope["query_string"])
         return self._query_params
 
     @property
@@ -77,6 +72,11 @@ class Request(Mapping):
             self._cookies = cookies
         return self._cookies
 
+    @property
+    def client(self) -> Address:
+        host, port = self._scope.get("client") or (None, None)
+        return Address(host=host, port=port)
+
     @property
     def session(self) -> dict:
         assert (
@@ -85,17 +85,56 @@ class Request(Mapping):
         return self._scope["session"]
 
     @property
-    def receive(self) -> Receive:
-        return self._receive
+    def database(self) -> typing.Any:  # pragma: no cover
+        assert (
+            "database" in self._scope
+        ), "DatabaseMiddleware must be installed to access request.database"
+        return self._scope["database"]
+
+    @property
+    def auth(self) -> typing.Any:
+        assert (
+            "auth" in self._scope
+        ), "AuthenticationMiddleware must be installed to access request.auth"
+        return self._scope["auth"]
+
+    @property
+    def user(self) -> typing.Any:
+        assert (
+            "user" in self._scope
+        ), "AuthenticationMiddleware must be installed to access request.user"
+        return self._scope["user"]
 
     def url_for(self, name: str, **path_params: typing.Any) -> str:
         router = self._scope["router"]
         url_path = router.url_path_for(name, **path_params)
         return url_path.make_absolute_url(base_url=self.url)
 
+
+async def empty_receive() -> Message:
+    raise RuntimeError("Receive channel has not been made available")
+
+
+class Request(HTTPConnection):
+    def __init__(self, scope: Scope, receive: Receive = empty_receive):
+        super().__init__(scope)
+        assert scope["type"] == "http"
+        self._receive = receive
+        self._stream_consumed = False
+        self._is_disconnected = False
+
+    @property
+    def method(self) -> str:
+        return self._scope["method"]
+
+    @property
+    def receive(self) -> Receive:
+        return self._receive
+
     async def stream(self) -> typing.AsyncGenerator[bytes, None]:
         if hasattr(self, "_body"):
             yield self._body
+            yield b""
             return
 
         if self._stream_consumed:
@@ -111,6 +150,7 @@ class Request(Mapping):
                 if not message.get("more_body", False):
                     break
             elif message["type"] == "http.disconnect":
+                self._is_disconnected = True
                 raise ClientDisconnect()
         yield b""
 
@@ -128,7 +168,7 @@ class Request(Mapping):
             self._json = json.loads(body)
         return self._json
 
-    async def form(self) -> dict:
+    async def form(self) -> FormData:
         if not hasattr(self, "_form"):
             assert (
                 parse_options_header is not None
@@ -139,14 +179,24 @@ class Request(Mapping):
                 multipart_parser = MultiPartParser(self.headers, self.stream())
                 self._form = await multipart_parser.parse()
             elif content_type == b"application/x-www-form-urlencoded":
-                from_parser = FormParser(self.headers, self.stream())
-                self._form = await from_parser.parse()
+                form_parser = FormParser(self.headers, self.stream())
+                self._form = await form_parser.parse()
             else:
-                self._form = {}
+                self._form = FormData()
         return self._form
 
     async def close(self) -> None:
         if hasattr(self, "_form"):
-            for item in self._form.values():
-                if hasattr(item, "close"):
-                    await item.close()  # type: ignore
+            await self._form.close()
+
+    async def is_disconnected(self) -> bool:
+        if not self._is_disconnected:
+            try:
+                message = await asyncio.wait_for(self._receive(), timeout=0.0000001)
+            except asyncio.TimeoutError as exc:
+                message = {}
+
+            if message.get("type") == "http.disconnect":
+                self._is_disconnected = True
+
+        return self._is_disconnected
