@@ -31,13 +31,16 @@ class Response:
 
     def __init__(
         self,
-        content: typing.Any,
+        content: typing.Any = None,
         status_code: int = 200,
         headers: dict = None,
         media_type: str = None,
         background: BackgroundTask = None,
     ) -> None:
-        self.body = self.render(content)
+        if content is None:
+            self.body = b""
+        else:
+            self.body = self.render(content)
         self.status_code = status_code
         if media_type is not None:
             self.media_type = media_type
@@ -60,11 +63,11 @@ class Response:
                 for k, v in headers.items()
             ]
             keys = [h[0] for h in raw_headers]
-            populate_content_length = b"content-length" in keys
-            populate_content_type = b"content-type" in keys
+            populate_content_length = b"content-length" not in keys
+            populate_content_type = b"content-type" not in keys
 
-        body = getattr(self, "body", None)
-        if body is not None and populate_content_length:
+        body = getattr(self, "body", b"")
+        if body and populate_content_length:
             content_length = str(len(body))
             raw_headers.append((b"content-length", content_length.encode("latin-1")))
 
@@ -107,7 +110,7 @@ class Response:
             cookie[key]["secure"] = True  # type: ignore
         if httponly:
             cookie[key]["httponly"] = True  # type: ignore
-        cookie_val = cookie.output(header="")
+        cookie_val = cookie.output(header="").strip()
         self.raw_headers.append((b"set-cookie", cookie_val.encode("latin-1")))
 
     def delete_cookie(self, key: str, path: str = "/", domain: str = None) -> None:
@@ -135,6 +138,39 @@ class PlainTextResponse(Response):
     media_type = "text/plain"
 
 
+class TemplateResponse(Response):
+    media_type = "text/html"
+
+    def __init__(
+        self,
+        template: typing.Any,
+        context: dict,
+        status_code: int = 200,
+        headers: dict = None,
+        media_type: str = None,
+        background: BackgroundTask = None,
+    ):
+        if "request" not in context:
+            raise ValueError('context must include a "request" key')
+        self.template = template
+        self.context = context
+        content = template.render(context)
+        super().__init__(content, status_code, headers, media_type, background)
+
+    async def __call__(self, receive: Receive, send: Send) -> None:
+        request = self.context["request"]
+        extensions = request.get("extensions", {})
+        if "http.response.template" in extensions:
+            await send(
+                {
+                    "type": "http.response.template",
+                    "template": self.template,
+                    "context": self.context,
+                }
+            )
+        await super().__call__(receive, send)
+
+
 class JSONResponse(Response):
     media_type = "application/json"
 
@@ -170,10 +206,12 @@ class StreamingResponse(Response):
         status_code: int = 200,
         headers: dict = None,
         media_type: str = None,
+        background: BackgroundTask = None,
     ) -> None:
         self.body_iterator = content
         self.status_code = status_code
         self.media_type = self.media_type if media_type is None else media_type
+        self.background = background
         self.init_headers(headers)
 
     async def __call__(self, receive: Receive, send: Send) -> None:
@@ -190,6 +228,9 @@ class StreamingResponse(Response):
             await send({"type": "http.response.body", "body": chunk, "more_body": True})
         await send({"type": "http.response.body", "body": b"", "more_body": False})
 
+        if self.background is not None:
+            await self.background()
+
 
 class FileResponse(Response):
     chunk_size = 4096
@@ -199,16 +240,20 @@ class FileResponse(Response):
         path: str,
         headers: dict = None,
         media_type: str = None,
+        background: BackgroundTask = None,
         filename: str = None,
         stat_result: os.stat_result = None,
+        method: str = None,
     ) -> None:
         assert aiofiles is not None, "'aiofiles' must be installed to use FileResponse"
         self.path = path
         self.status_code = 200
         self.filename = filename
+        self.send_header_only = method is not None and method.upper() == "HEAD"
         if media_type is None:
             media_type = guess_type(filename or path)[0] or "text/plain"
         self.media_type = media_type
+        self.background = background
         self.init_headers(headers)
         if self.filename is not None:
             content_disposition = 'attachment; filename="{}"'.format(self.filename)
@@ -217,14 +262,22 @@ class FileResponse(Response):
         if stat_result is not None:
             self.set_stat_headers(stat_result)
 
-    def set_stat_headers(self, stat_result: os.stat_result) -> None:
+    @classmethod
+    def get_stat_headers(cls, stat_result: os.stat_result) -> typing.Dict[str, str]:
         content_length = str(stat_result.st_size)
         last_modified = formatdate(stat_result.st_mtime, usegmt=True)
         etag_base = str(stat_result.st_mtime) + "-" + str(stat_result.st_size)
         etag = hashlib.md5(etag_base.encode()).hexdigest()
-        self.headers.setdefault("content-length", content_length)
-        self.headers.setdefault("last-modified", last_modified)
-        self.headers.setdefault("etag", etag)
+        return {
+            "content-length": content_length,
+            "last-modified": last_modified,
+            "etag": etag,
+        }
+
+    def set_stat_headers(self, stat_result: os.stat_result) -> None:
+        stat_headers = self.get_stat_headers(stat_result)
+        for name, value in stat_headers.items():
+            self.headers.setdefault(name, value)
 
     async def __call__(self, receive: Receive, send: Send) -> None:
         if self.stat_result is None:
@@ -244,15 +297,20 @@ class FileResponse(Response):
                 "headers": self.raw_headers,
             }
         )
-        async with aiofiles.open(self.path, mode="rb") as file:
-            more_body = True
-            while more_body:
-                chunk = await file.read(self.chunk_size)
-                more_body = len(chunk) == self.chunk_size
-                await send(
-                    {
-                        "type": "http.response.body",
-                        "body": chunk,
-                        "more_body": more_body,
-                    }
-                )
+        if self.send_header_only:
+            await send({"type": "http.response.body"})
+        else:
+            async with aiofiles.open(self.path, mode="rb") as file:
+                more_body = True
+                while more_body:
+                    chunk = await file.read(self.chunk_size)
+                    more_body = len(chunk) == self.chunk_size
+                    await send(
+                        {
+                            "type": "http.response.body",
+                            "body": chunk,
+                            "more_body": more_body,
+                        }
+                    )
+        if self.background is not None:
+            await self.background()
